
module lock(clk, reset_in, b0_in, b1_in, out, hex_display);
	input clk, reset_in, b0_in,b1_in;
	output out; output [3:0] hex_display;

   // synchronize push buttons, convert to pulses

   wire reset, b0, b1;
   button b_reset(clk,reset_in,reset);
   button b_0(clk,b0_in,b0);
   button b_1(clk,b1_in,b1);

   // implement state transition diagram

  parameter S_RESET = 0;
  parameter S_0 = 1;
  parameter S_01 = 2;
  parameter S_010 =3;
  parameter S_0101 = 4;
  parameter S_01011 = 5;

  reg [2:0] state;

  always @ (posedge clk)
  begin
    if (reset) state <= S_RESET;
    else case (state)
      S_RESET: state <= b0 ? S_0   : ( b1 ? S_RESET : state );
      S_0:     state <= b0 ? S_0   : ( b1 ? S_01 : state );
      S_01:    state <= b0 ? S_010 : ( b1 ? S_RESET : state );
      S_010:   state <= b0 ? S_0   : ( b1 ? S_0101 : state );
      S_0101:  state <= b0 ? S_010 : ( b1 ? S_01011 : state );
      S_01011: state <= b0 ? S_0   : ( b1 ? S_RESET : state );
      default: state <= S_RESET; // handle unused states
    endcase
  end

  // it's a Moore machine! Output only depends on current state
  assign out = (state == S_01011);
  assign hex_display = {1'b0, state};

endmodule

/////////////////////////////////////////////////////////////////////////////

module button(clk, in, out);
  input clk;
  input in;
  output out;

  // this module outputs one pulse for each button press

  reg r1,r2,r3;
  always @ (posedge clk)
  begin
    r1 <= in;
    r2 <= r1;
    r3 <= r2;
  end
  assign out = ~r3 & r2;

endmodule
